﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("MutationPerformanceAnalyzer", "An operator that analyzes the performance of mutation.")]
  [StorableClass]
  public class MutationPerformanceAnalyzer : SingleSuccessorOperator, IAnalyzer {
    private const string ResultsParameterName = "Results";
    private const string GenerationsParameterName = "Generations";

    #region IAnalyzer Members
    public bool EnabledByDefault {
      get { return true; }
    }
    #endregion


    #region Parameter properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    public ILookupParameter<IntValue> GenerationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters[GenerationsParameterName]; }
    }
    public ILookupParameter<DoubleValue> QualityAfterCrossoverParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["QualityAfterCrossover"]; }
    }
    public ILookupParameter<DoubleValue> QualityAfterMutationParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["QualityAfterMutation"]; }
    }
    #endregion

    #region Properties
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }
    #endregion

    ScatterPlot plot;
    DataRow dtRow;
    int cnt = 0;

    [StorableConstructor]
    private MutationPerformanceAnalyzer(bool deserializing) : base(deserializing) { }
    private MutationPerformanceAnalyzer(MutationPerformanceAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public MutationPerformanceAnalyzer()
      : base() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection where the analysis values should be stored."));
      Parameters.Add(new LookupParameter<IntValue>(GenerationsParameterName, "Nr of generations."));

      Parameters.Add(new LookupParameter<DoubleValue>("QualityAfterCrossover", "The evaluated quality of the child solution."));
      QualityAfterCrossoverParameter.ActualName = "TSPTourLength";

      Parameters.Add(new LookupParameter<DoubleValue>("QualityAfterMutation", "The evaluated quality of the child solution."));
      QualityAfterMutationParameter.ActualName = "TSPTourLengthM";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MutationPerformanceAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      Point2D<double> curPoint;
      var qualityCX = QualityAfterCrossoverParameter.ActualValue.Value;
      var qualityM = QualityAfterMutationParameter.ActualValue.Value;

      curPoint = new Point2D<double>(cnt++, qualityCX - qualityM);

      string curGenStr = GenerationsParameter.ActualValue.Value.ToString();
      ScatterPlotDataRow row;

      if (!Results.ContainsKey("Mutation Scatter Plot")) {
        InitializePlot();
        Results.Add(new Result("Mutation Scatter Plot", plot));
        Results.Add(new Result("Mutation Scatter Plot History", new ScatterPlotHistory()));
        cnt = 0;

        DataTable dt = new DataTable("Average Mutatioin Performance");
        dtRow = new DataRow("Average Mutation Performance per Generation");
        dt.Rows.Add(dtRow);
        Results.Add(new Result("Average Mutation Performance", dt));
      }

      if (!plot.Rows.ContainsKey(curGenStr)) {
        if (GenerationsParameter.ActualValue.Value != 0) {
          if (plot.Rows.ContainsKey((GenerationsParameter.ActualValue.Value - 1).ToString())) {
            double avg = plot.Rows[(GenerationsParameter.ActualValue.Value - 1).ToString()].Points.Average(x => x.Y);
            dtRow.Values.Add(avg);
            ((ScatterPlotHistory)Results["Mutation Scatter Plot History"].Value).Add(plot);
          }
          InitializePlot();
          Results["Mutation Scatter Plot"].Value = plot;
          cnt = 0;
        }

        var points = new List<Point2D<double>>();
        points.Add(curPoint);
        row = new ScatterPlotDataRow(curGenStr, null, points);
        row.VisualProperties.PointStyle = ScatterPlotDataRowVisualProperties.ScatterPlotDataRowPointStyle.Circle;
        row.VisualProperties.PointSize = 5;
        plot.Rows.Add(row);
      } else {
        plot.Rows[curGenStr].Points.Add(curPoint);
      }

      return base.Apply();
    }

    private void InitializePlot() {
      plot = new ScatterPlot("Mutation Performance", null);
      plot.VisualProperties.XAxisTitle = "Solution Index";
      plot.VisualProperties.YAxisTitle = "Absolut Quality Difference";
    }
  }
}
